@@ -155,7 +155,7 @@ impl<'a> PythonResourceIterator<'a> {
         let mut paths = resources
             .iter()
             .map(|file| PathEntry {
-                path: file.path.clone(),
+                path: file.path().to_path_buf(),
                 file_emitted: false,
                 non_file_emitted: false,
             })
@@ -164,7 +164,7 @@ impl<'a> PythonResourceIterator<'a> {
 
         let mut path_content_overrides = FileManifest::default();
         for resource in resources {
-            path_content_overrides.add_file_entry(&resource.path, resource.entry.clone())?;
+            path_content_overrides.add_file_entry(resource.path(), resource.entry().clone())?;
         }
 
         Ok(PythonResourceIterator {
@@ -183,7 +183,7 @@ impl<'a> PythonResourceIterator<'a> {
 
     fn resolve_is_executable(&self, path: &Path) -> bool {
         match self.path_content_overrides.get(path) {
-            Some(file) => file.executable,
+            Some(file) => file.is_executable(),
             None => {
                 if let Ok(metadata) = path.metadata() {
                     is_executable(&metadata)
@@ -196,7 +196,7 @@ impl<'a> PythonResourceIterator<'a> {
 
     fn resolve_file_data(&self, path: &Path) -> FileData {
         match self.path_content_overrides.get(path) {
-            Some(file) => file.data.clone(),
+            Some(file) => file.file_data().clone(),
             None => FileData::Path(path.to_path_buf()),
         }
     }
@@ -234,7 +234,7 @@ impl<'a> PythonResourceIterator<'a> {
 
         if let Some((metadata_path, location)) = distribution_info {
             let data = if let Some(file) = self.path_content_overrides.get(&metadata_path) {
-                if let Ok(data) = file.data.resolve() {
+                if let Ok(data) = file.resolve_content() {
                     data
                 } else {
                     return None;
@@ -552,13 +552,13 @@ impl<'a> Iterator for PythonResourceIterator<'a> {
                     .expect("unable to strip path prefix")
                     .to_path_buf();
 
-                let f = File {
-                    path: rel_path,
-                    entry: FileEntry {
-                        executable: self.resolve_is_executable(&self.paths[0].path),
-                        data: self.resolve_file_data(&self.paths[0].path),
-                    },
-                };
+                let f = File::new(
+                    rel_path,
+                    FileEntry::new_from_data(
+                        self.resolve_file_data(&self.paths[0].path),
+                        self.resolve_is_executable(&self.paths[0].path),
+                    ),
+                );
 
                 return Some(Ok(f.into()));
             }
@@ -703,7 +703,10 @@ mod tests {
     use {
         super::*,
         once_cell::sync::Lazy,
-        std::fs::{create_dir_all, write},
+        std::{
+            convert::TryFrom,
+            fs::{create_dir_all, write},
+        },
     };
 
     const DEFAULT_CACHE_TAG: &str = "cpython-37";
@@ -743,13 +746,10 @@ mod tests {
 
         assert_eq!(
             resources[0],
-            File {
-                path: PathBuf::from("acme/__init__.py"),
-                entry: FileEntry {
-                    executable: false,
-                    data: acme_path.join("__init__.py").into(),
-                }
-            }
+            File::new(
+                "acme/__init__.py",
+                FileEntry::try_from(acme_path.join("__init__.py"))?
+            )
             .into()
         );
         assert_eq!(
@@ -766,13 +766,10 @@ mod tests {
         );
         assert_eq!(
             resources[2],
-            File {
-                path: PathBuf::from("acme/a/__init__.py"),
-                entry: FileEntry {
-                    executable: false,
-                    data: acme_a_path.join("__init__.py").into(),
-                }
-            }
+            File::new(
+                "acme/a/__init__.py",
+                FileEntry::try_from(acme_a_path.join("__init__.py"))?
+            )
             .into()
         );
         assert_eq!(
@@ -789,13 +786,10 @@ mod tests {
         );
         assert_eq!(
             resources[4],
-            File {
-                path: PathBuf::from("acme/a/foo.py"),
-                entry: FileEntry {
-                    executable: false,
-                    data: acme_a_path.join("foo.py").into(),
-                }
-            }
+            File::new(
+                "acme/a/foo.py",
+                FileEntry::try_from(acme_a_path.join("foo.py"))?
+            )
             .into()
         );
         assert_eq!(
@@ -812,13 +806,10 @@ mod tests {
         );
         assert_eq!(
             resources[6],
-            File {
-                path: PathBuf::from("acme/bar/__init__.py"),
-                entry: FileEntry {
-                    executable: false,
-                    data: acme_bar_path.join("__init__.py").into(),
-                }
-            }
+            File::new(
+                "acme/bar/__init__.py",
+                FileEntry::try_from(acme_bar_path.join("__init__.py"))?
+            )
             .into()
         );
         assert_eq!(
@@ -1850,20 +1841,8 @@ mod tests {
     #[test]
     fn test_memory_resources() -> Result<()> {
         let inputs = vec![
-            File {
-                path: PathBuf::from("foo/__init__.py"),
-                entry: FileEntry {
-                    executable: false,
-                    data: vec![0].into(),
-                },
-            },
-            File {
-                path: PathBuf::from("foo/bar.py"),
-                entry: FileEntry {
-                    executable: true,
-                    data: vec![1].into(),
-                },
-            },
+            File::new("foo/__init__.py", vec![0]),
+            File::new("foo/bar.py", FileEntry::new_from_data(vec![1], true)),
         ];
 
         let resources = PythonResourceIterator::from_data_locations(
@@ -1876,17 +1855,7 @@ mod tests {
         .collect::<Result<Vec<_>>>()?;
 
         assert_eq!(resources.len(), 4);
-        assert_eq!(
-            resources[0],
-            File {
-                path: PathBuf::from("foo/__init__.py"),
-                entry: FileEntry {
-                    executable: false,
-                    data: vec![0].into(),
-                }
-            }
-            .into()
-        );
+        assert_eq!(resources[0], File::new("foo/__init__.py", vec![0]).into());
         assert_eq!(
             resources[1],
             PythonModuleSource {
@@ -1901,14 +1870,7 @@ mod tests {
         );
         assert_eq!(
             resources[2],
-            File {
-                path: PathBuf::from("foo/bar.py"),
-                entry: FileEntry {
-                    executable: true,
-                    data: vec![1].into(),
-                }
-            }
-            .into()
+            File::new("foo/bar.py", FileEntry::new_from_data(vec![1], true)).into()
         );
         assert_eq!(
             resources[3],
