@@ -14,7 +14,10 @@ use {
         resource_collection::PythonResourceAddCollectionContext,
     },
     anyhow::Result,
-    std::{collections::HashMap, convert::TryFrom},
+    std::{
+        collections::{HashMap, HashSet},
+        convert::TryFrom,
+    },
     tugger_licensing::LicenseFlavor,
 };
 
@@ -172,6 +175,9 @@ pub struct PythonPackagingPolicy {
 
     /// Whether to write Python bytecode at optimization level 2.
     bytecode_optimize_level_two: bool,
+
+    /// Python modules for which bytecode should not be generated by default.
+    no_bytecode_modules: HashSet<String>,
 }
 
 impl Default for PythonPackagingPolicy {
@@ -195,6 +201,7 @@ impl Default for PythonPackagingPolicy {
             bytecode_optimize_level_zero: true,
             bytecode_optimize_level_one: false,
             bytecode_optimize_level_two: false,
+            no_bytecode_modules: HashSet::new(),
         }
     }
 }
@@ -417,6 +424,17 @@ impl PythonPackagingPolicy {
             .push(extension.to_string());
     }
 
+    /// Register a Python module as one that should not generate bytecode.
+    ///
+    /// When source modules matching names registered with this function are added,
+    /// their default settings for adding bytecode will always be false.
+    ///
+    /// It is still possible to force bytecode generation by setting the add context
+    /// fields to true or explicitly adding a bytecode resource.
+    pub fn register_no_bytecode_module(&mut self, name: &str) {
+        self.no_bytecode_modules.insert(name.to_string());
+    }
+
     /// Derive a `PythonResourceAddCollectionContext` for a resource using current settings.
     ///
     /// The returned object essentially says how the resource should be added
@@ -441,14 +459,45 @@ impl PythonPackagingPolicy {
         let location = self.resources_location.clone();
         let location_fallback = self.resources_location_fallback.clone();
 
+        let optimize_level_zero = match resource {
+            PythonResource::ModuleSource(module) => {
+                if self.no_bytecode_modules.contains(&*module.name) {
+                    false
+                } else {
+                    self.bytecode_optimize_level_zero
+                }
+            }
+            _ => self.bytecode_optimize_level_zero,
+        };
+        let optimize_level_one = match resource {
+            PythonResource::ModuleSource(module) => {
+                if self.no_bytecode_modules.contains(&*module.name) {
+                    false
+                } else {
+                    self.bytecode_optimize_level_one
+                }
+            }
+            _ => self.bytecode_optimize_level_one,
+        };
+        let optimize_level_two = match resource {
+            PythonResource::ModuleSource(module) => {
+                if self.no_bytecode_modules.contains(&*module.name) {
+                    false
+                } else {
+                    self.bytecode_optimize_level_two
+                }
+            }
+            _ => self.bytecode_optimize_level_two,
+        };
+
         PythonResourceAddCollectionContext {
             include,
             location,
             location_fallback,
             store_source,
-            optimize_level_zero: self.bytecode_optimize_level_zero,
-            optimize_level_one: self.bytecode_optimize_level_one,
-            optimize_level_two: self.bytecode_optimize_level_two,
+            optimize_level_zero,
+            optimize_level_one,
+            optimize_level_two,
         }
     }
 
@@ -634,24 +683,14 @@ impl PythonPackagingPolicy {
 
 #[cfg(test)]
 mod tests {
-    use {
-        super::*,
-        std::path::PathBuf,
-        tugger_file_manifest::{File, FileEntry},
-    };
+    use {super::*, tugger_file_manifest::File};
 
     #[test]
     fn test_add_collection_context_file() -> Result<()> {
         let mut policy = PythonPackagingPolicy::default();
         policy.include_file_resources = false;
 
-        let file = File {
-            path: PathBuf::from("foo.py"),
-            entry: FileEntry {
-                executable: false,
-                data: vec![42].into(),
-            },
-        };
+        let file = File::new("foo.py", vec![42]);
 
         let add_context = policy.derive_add_collection_context(&file.clone().into());
         assert!(!add_context.include);
