@@ -149,7 +149,7 @@ pub struct WheelBuilder {
     manifest: FileManifest,
 
     /// The modified time to write for files in the wheel archive.
-    modified_time: time::Tm,
+    modified_time: time::OffsetDateTime,
 }
 
 impl WheelBuilder {
@@ -165,7 +165,7 @@ impl WheelBuilder {
             generator: "rust-python-packaging".to_string(),
             root_is_purelib: false,
             manifest: FileManifest::default(),
-            modified_time: time::now_utc(),
+            modified_time: time::OffsetDateTime::now_utc(),
         }
     }
 
@@ -261,12 +261,12 @@ impl WheelBuilder {
     }
 
     /// Obtain the modified time for files in the wheel archive.
-    pub fn modified_time(&self) -> time::Tm {
+    pub fn modified_time(&self) -> time::OffsetDateTime {
         self.modified_time
     }
 
     /// Set the modified time for files in the wheel archive.
-    pub fn set_modified_time(&mut self, v: time::Tm) {
+    pub fn set_modified_time(&mut self, v: time::OffsetDateTime) {
         self.modified_time = v;
     }
 
@@ -285,7 +285,7 @@ impl WheelBuilder {
     /// Add a file to the wheel at the given path.
     ///
     /// No validation of the path is performed.
-    pub fn add_file(&mut self, path: impl AsRef<Path>, file: FileEntry) -> Result<()> {
+    pub fn add_file(&mut self, path: impl AsRef<Path>, file: impl Into<FileEntry>) -> Result<()> {
         self.manifest.add_file_entry(path, file)?;
 
         Ok(())
@@ -295,7 +295,11 @@ impl WheelBuilder {
     ///
     /// Attempts to add the `RECORD` file will work. However, the content will be
     /// ignored and regenerated as part of wheel building.
-    pub fn add_file_dist_info(&mut self, path: impl AsRef<Path>, file: FileEntry) -> Result<()> {
+    pub fn add_file_dist_info(
+        &mut self,
+        path: impl AsRef<Path>,
+        file: impl Into<FileEntry>,
+    ) -> Result<()> {
         self.manifest
             .add_file_entry(self.dist_info_path().join(path), file)?;
 
@@ -314,7 +318,7 @@ impl WheelBuilder {
         &mut self,
         destination: impl ToString,
         path: impl AsRef<Path>,
-        file: FileEntry,
+        file: impl Into<FileEntry>,
     ) -> Result<()> {
         self.manifest.add_file_entry(
             PathBuf::from(format!(
@@ -356,8 +360,7 @@ impl WheelBuilder {
             .iter_entries()
             .map(|(path, entry)| {
                 let content = entry
-                    .data
-                    .resolve()
+                    .resolve_content()
                     .with_context(|| format!("resolving content for {}", path.display()))?;
 
                 let mut digest = sha2::Sha256::new();
@@ -414,10 +417,7 @@ impl WheelBuilder {
         if !m.has_path(self.dist_info_path().join("WHEEL")) {
             m.add_file_entry(
                 self.dist_info_path().join("WHEEL"),
-                FileEntry {
-                    data: self.derive_wheel_file().as_bytes().into(),
-                    executable: false,
-                },
+                self.derive_wheel_file().as_bytes(),
             )?;
         }
 
@@ -425,10 +425,7 @@ impl WheelBuilder {
         if !m.has_path(self.dist_info_path().join("METADATA")) {
             m.add_file_entry(
                 self.dist_info_path().join("METADATA"),
-                FileEntry {
-                    data: self.derive_metadata_file().as_bytes().into(),
-                    executable: false,
-                },
+                self.derive_metadata_file().as_bytes(),
             )?;
         }
 
@@ -438,14 +435,9 @@ impl WheelBuilder {
 
         m.add_file_entry(
             self.dist_info_path().join("RECORD"),
-            FileEntry {
-                data: self
-                    .derive_record_file(&m)
-                    .context("deriving RECORD file")?
-                    .as_bytes()
-                    .into(),
-                executable: false,
-            },
+            self.derive_record_file(&m)
+                .context("deriving RECORD file")?
+                .as_bytes(),
         )?;
 
         Ok(m)
@@ -464,12 +456,14 @@ impl WheelBuilder {
         let mut files = m.iter_files().collect::<Vec<_>>();
         let dist_info_path = self.dist_info_path();
         files.sort_by(|a, b| {
-            if a.path.starts_with(&dist_info_path) && !b.path.starts_with(&dist_info_path) {
+            if a.path().starts_with(&dist_info_path) && !b.path().starts_with(&dist_info_path) {
                 Ordering::Greater
-            } else if b.path.starts_with(&dist_info_path) && !a.path.starts_with(&dist_info_path) {
+            } else if b.path().starts_with(&dist_info_path)
+                && !a.path().starts_with(&dist_info_path)
+            {
                 Ordering::Less
             } else {
-                a.path.cmp(&b.path)
+                a.path().cmp(b.path())
             }
         });
 
@@ -477,25 +471,31 @@ impl WheelBuilder {
 
         for file in files.into_iter() {
             let options = zip::write::FileOptions::default()
-                .unix_permissions(if file.entry.executable {
+                .unix_permissions(if file.entry().is_executable() {
                     0o0755
                 } else {
                     0o0644
                 })
                 .last_modified_time(
-                    zip::DateTime::from_time(self.modified_time)
-                        .map_err(|_| anyhow!("could not convert time to zip::DateTime"))?,
+                    zip::DateTime::from_date_and_time(
+                        self.modified_time.year() as u16,
+                        self.modified_time.month(),
+                        self.modified_time.day(),
+                        self.modified_time.hour(),
+                        self.modified_time.minute(),
+                        self.modified_time.second(),
+                    )
+                    .map_err(|_| anyhow!("could not convert time to zip::DateTime"))?,
                 );
 
-            zf.start_file(format!("{}", file.path.display()), options)?;
+            zf.start_file(format!("{}", file.path().display()), options)?;
             zf.write_all(
                 &file
-                    .entry
-                    .data
-                    .resolve()
-                    .with_context(|| format!("resolving content of {}", file.path.display()))?,
+                    .entry()
+                    .resolve_content()
+                    .with_context(|| format!("resolving content of {}", file.path().display()))?,
             )
-            .with_context(|| format!("writing zip member {}", file.path.display()))?;
+            .with_context(|| format!("writing zip member {}", file.path().display()))?;
         }
 
         zf.finish().context("finishing zip file")?;
@@ -537,25 +537,19 @@ mod test {
 
         let m = builder.build_file_manifest()?;
         assert_eq!(m.iter_entries().count(), 3);
-        assert_eq!(m.get("my_package-0.1.dist-info/WHEEL"), Some(&FileEntry {
-            data: b"Wheel-Version: 1.0\nGenerator: rust-python-packaging\nRoot-Is-Purelib: false\nTag: py3-none-any\n".as_ref().into(),
-            executable: false
-        }));
+        assert_eq!(m.get("my_package-0.1.dist-info/WHEEL"),
+                   Some(&b"Wheel-Version: 1.0\nGenerator: rust-python-packaging\nRoot-Is-Purelib: false\nTag: py3-none-any\n".as_ref().into()));
         assert_eq!(
             m.get("my_package-0.1.dist-info/METADATA"),
-            Some(&FileEntry {
-                data: b"Metadata-Version: 2.1\nName: my-package\nVersion: 0.1\n"
+            Some(
+                &b"Metadata-Version: 2.1\nName: my-package\nVersion: 0.1\n"
                     .as_ref()
-                    .into(),
-                executable: false
-            })
+                    .into()
+            )
         );
         assert_eq!(
             m.get("my_package-0.1.dist-info/RECORD"),
-            Some(&FileEntry {
-                data: b"my_package-0.1.dist-info/METADATA,sha256=sXUNNYpfVReu7VHhVzSbKiT5ciO4Fwcwm7icBNiYn3Y,52\nmy_package-0.1.dist-info/WHEEL,sha256=76DhAzqMvlOgtCOiUNpWcD643b1CXd507uRH1hq6fQw,93\nmy_package-0.1.dist-info/RECORD,,\n".as_ref().into(),
-                executable: false
-            })
+            Some(&b"my_package-0.1.dist-info/METADATA,sha256=sXUNNYpfVReu7VHhVzSbKiT5ciO4Fwcwm7icBNiYn3Y,52\nmy_package-0.1.dist-info/WHEEL,sha256=76DhAzqMvlOgtCOiUNpWcD643b1CXd507uRH1hq6fQw,93\nmy_package-0.1.dist-info/RECORD,,\n".as_ref().into())
         );
 
         Ok(())
@@ -598,21 +592,12 @@ mod test {
     fn custom_wheel_file() -> Result<()> {
         let mut builder = WheelBuilder::new("my-package", "0.1");
 
-        builder.add_file_dist_info(
-            "WHEEL",
-            FileEntry {
-                data: vec![42].into(),
-                executable: false,
-            },
-        )?;
+        builder.add_file_dist_info("WHEEL", vec![42])?;
 
         let m = builder.build_file_manifest()?;
         assert_eq!(
             m.get("my_package-0.1.dist-info/WHEEL"),
-            Some(&FileEntry {
-                data: vec![42].into(),
-                executable: false
-            })
+            Some(&vec![42].into())
         );
 
         Ok(())
@@ -622,21 +607,12 @@ mod test {
     fn custom_metadata_file() -> Result<()> {
         let mut builder = WheelBuilder::new("my-package", "0.1");
 
-        builder.add_file_dist_info(
-            "METADATA",
-            FileEntry {
-                data: vec![42].into(),
-                executable: false,
-            },
-        )?;
+        builder.add_file_dist_info("METADATA", vec![42])?;
 
         let m = builder.build_file_manifest()?;
         assert_eq!(
             m.get("my_package-0.1.dist-info/METADATA"),
-            Some(&FileEntry {
-                data: vec![42].into(),
-                executable: false
-            })
+            Some(&vec![42].into())
         );
 
         Ok(())
@@ -646,22 +622,12 @@ mod test {
     fn add_file_data() -> Result<()> {
         let mut builder = WheelBuilder::new("my-package", "0.1");
 
-        builder.add_file_data(
-            "purelib",
-            "__init__.py",
-            FileEntry {
-                data: vec![42].into(),
-                executable: false,
-            },
-        )?;
+        builder.add_file_data("purelib", "__init__.py", vec![42])?;
 
         let m = builder.build_file_manifest()?;
         assert_eq!(
             m.get("my_package-0.1.data/purelib/__init__.py"),
-            Some(&FileEntry {
-                data: vec![42].into(),
-                executable: false
-            })
+            Some(&vec![42].into())
         );
 
         Ok(())
